//
//		Definition of the plant class
//
#ifndef THE_PLANT_CLASS_H

#define THE_PLANT_CLASS_H

#include "ldwTypes.h"
#include "theGameState.h"
#include "theGraphicsManager.h"

#define numSecondsAutoPause			31000

const float cNurseryScaleFactor   = 0.6f;
enum
{
	eNorth      = 1,
	eNorthEast  = 1 << 1,
	eEast       = 1 << 2,
	eSouthEast  = 1 << 3,
	eSouth      = 1 << 4,
	eSouthWest  = 1 << 5,
	eWest       = 1 << 6,
	eNorthWest  = 1 << 7,

	eBranchPoint = 1 << 8

	,eNoTarget = -1

    ,eOuterBranch = 1
};

typedef struct
{
	int branchX;
	int branchY;
	int branchType;
}
BranchPointType;

typedef struct
{
	int budX;
	int budY;
	int budType;
}
BudPointType;

typedef struct
{
	int     partId;
	int     partType;
	ImageId	imageId;
	ImageId glowMaskId;
	ImageId deadId;

	int sourceX;
	int sourceY;
	int width;
	int height;

	int baseX;
	int baseY;

	int branchCount;
	BranchPointType branch[eMaxPlantBranches];

	int budCount;
	BudPointType    bud[eMaxFlowers];
}
PlantPartDefType;

typedef enum 
{
	eNoFlower,

	eFlowerBud = eNoFlower,

	eSmallFlowerYellow1, eCactusFlowerPink,	eDaisyWhite, eOrchidBlack, eNormalFlowerBlue,		// 1 - 5

	eLotusPink,  eRoseRed,  eNormalFlowerViola,  eJalapa, eRoseBlueLarge,			// 6 - 10

	eHibiscus, eSunFlower, eBegonia, eGoldenBerry, eBerryBlack,								// 11 - 15  	

	eApple, eLemon, eAnthuriumRed, eOrchidWhite, eTiareTahitiWhite, 							// 16 - 20

	eTigerLilly, eBerryRed, eLime,															// 21 - 23

	eFlowerNextId,

	eFlowerIdMax = eFlowerNextId - 1
}
FlowerType;

typedef struct
{
	FlowerType id;
	int        flowerValue;

	ImageId    imageId;
	ImageId	   maskId;
	int        imageIndex;
	int        centerOffsetX;
	int        centerOffsetY;
}
FlowerDefType;

class ldwGameWindow;

class thePlantClass
{
public:

	thePlantClass(PlantType *plant);
	virtual ~thePlantClass();

	virtual void UpdateAge(ldwUINT32 currentTime);
	virtual void HealthTax();
	virtual void ChanceOfInfestation();
	virtual bool PlantIsForSale();

	virtual float GetAgeUntilTeen();
	virtual float GetAgeUntilAdult();

	void DrawForNursery(int x,int y,int magicFlowerType = 0);
	void DrawForSeedScene(int x,int y,int flowerId);

	virtual void Draw(int plantBaseX,int plantBaseY,bool enablePruning = true);

	virtual void DrawSprout(int x,int y,float scale) {};
	virtual void DrawTeen(int x,int y,float scale)   {};
	virtual void DrawAdult(int x,int y,float scale)  {};

	virtual void DrawPart(int partId,PlantPartDefType *parts,int x,int y,float scale,bool isDead = false,bool addGlow = false);
	virtual bool DrawPart(PlantPartDefType *thePart,int x,int y,float scale,bool isDead = false,bool addGlow = false);
	virtual void DrawParts(int sectionIndex,PlantPartDefType *parts,int x,int y,float scale,float parentAge,bool addGlow = false);

	virtual void DisablePruning();
	virtual bool IsPrunePoint();
	virtual bool Prune();

	virtual void AddFlowers(int baseCount,int extraCount);

	ldwPoint GetFlowerPoint();

	int GetFlowerCount();

	bool HasDeadBranch();

	void IncreaseMaxGrowth(int howMuch);
	void DecreaseMaxGrowth(int howMuch);

	int  GetTimesFertilized();

	bool PlantIsRuined();
	bool PlantWasPruned();


protected:

	void DrawScaled(int x,int y,float scale);

	bool DrawFlower(int x,int y,int orientation,float scale,bool addGlow = false);

	void PruneBranches(int branchStartIndex,int branchCount,int maxToPrune);
	void CheckLimits();
	
	FlowerDefType *GetFlowerDef(int flowerId);

	bool AddFlower();

	int  virtual GetBaseValue() = 0;

	float virtual GetMinScale();

	int  FindAvailableBudPoint(PlantSectionType *branch);
	bool BudPointAvailable(PlantSectionType *branch,PlantPartDefType *branchDef,int whichBudPoint);

	void UpdateValue();

	int CountAddedFlowers();
	int CountPrunedFlowers();

	void UpdateGlowLevel();
	void CountParts(PlantSectionType *part,int &totalCount,int &userPrunedCount,bool partWasPruned);

	void MaybeKillBranches();
	void MarkSubBranches(PlantSectionType *branch,PlantSectionStatus status);

	bool HasDeadSubBranch(PlantSectionType *branch);

	PlantType        *thePlant;
	PlantPartDefType *mParts;

	theGameState       *gs;
	theGraphicsManager *gm;
	ldwGameWindow      *gw;

	bool   mGlowing;
	bool   mEnablePruning;
	bool   mTargetFound;

	int    mPruneSectionTarget;
	int    mPruneFlowerTarget;

	int    mNextPruneSectionTarget;
	int    mNextPruneFlowerTarget;

	float  mGlowLevel;
	int    mGlowAngle;

	ldwPoint mCursorPt;

	bool	mAnimatingPrune;

	int     mPrunedSection;
	int     mPrunedFlower;

	ldwPoint mPrunePt;

	float   mPruneYDelta;
	float   mPruneScale;

	PlantPartDefType *FindPart(int partId,PlantPartDefType *parts);
};
#endif