//
//		Definition of the main game class
//
#ifndef THE_SOUND_MANAGER_H
#define THE_SOUND_MANAGER_H

#include "ldwSound.h"
#include "ldwGameWindow.h"

//class ldwButton;
class ldwSoundTrack;
class ldwSoundFx;
class theGameState;
//class ldwTextControl;

//CLASS DEFINES
#define cMaxNumSoundFx 100
#define cNumOfSoundFiles eNoSnd + 5
//...typedefs etc
typedef enum  //the enums are formed from the exact filenames with the following rules:
			  //'e' + [CAP]Filename + 'Snd'
{
	eStartSnds,//this has to be the first enum
	eAngelSnd,
	eDirtSnd,
	eSpraySnd,
	eDripSnd,
	ePruneSnd,
	eFountain_loop,
	eGate_close,
	eGate_opening,
	eLadybug_files,
	eMoving_clay_pots,
	eNursery_loop,
	eRollover_1,
	eRollover_2,
	eRollover_3,
	eRustle_1,
	eRustle_2,
	eRustle_3,
	eSeed_handling,
	eSeed_movement,
	eSeed_planting,
	eSnip_1,
	eSnip_2,
	eSnip_3,
	eTool_swap,
	eWatering_can,
	eWindchimes_1,
	eWindchimes_2,
	eBirds,
	eCash_register,
	eCatch_a_bug,
	eClick_1,
	eClick_2,
	eClick_3,
	eClose_trash_bin,
	eCoin,
	eCross_breed_chime,
	eCrowd_murmer,
	eDirt_1,
	eDirt_2,
	eFootsteps,
	eBirds2,
	eBuddhist_bell,
	eBug_move,
	eBug_spray,
	eBug_zapper,
	eButterfly,
	eCricket_loop,
	eCross_breed_chime_3,
	eGentle_fountain_loop,
	eInsect_buzz,
	eMedical_drip,
	eWind_trees,
	eWindchimes_4,

	eNoSnd //eNoSnd MUST be the last sound enum additional snds must be placed after it

} SoundNameEnum;	

typedef enum
{
	eNoSpecialEffect,
	eEcho,
	eReverb
	
} SpecialEffectEnum;	


typedef struct 
{	
	SoundNameEnum soundType;
	SpecialEffectEnum specialEffect;
	bool looping;
	bool exists;
	ldwSoundFx *soundFxPtr;
} SoundFxType;
//END DEFINES

//BEGIN CLASS DECLARATION
class theSoundManager 
{
public:

	static theSoundManager * Get();
	static void Destroy();

	theSoundManager();
	~theSoundManager();
	int RegisterSound(SoundNameEnum whichSound);
	int RegisterSound(SoundNameEnum whichSound, bool isLooping);

	ldwSoundFx *GetSnd(SoundNameEnum whichSound);

	void PlaySnd(SoundNameEnum whichSound);
	void PlaySnd(SoundNameEnum whichSound, bool isLooping);
	void PlaySnd(SoundNameEnum whichSound, float sndVol);
	void PlaySnd(SoundNameEnum whichSound, bool isLooping, float sndVol);

	void PlaySnd(int whichIntSound);
	void PlaySnd(int whichIntSound, bool isLooping);
	void PlaySnd(int whichIntSound, float sndVol);
	void PlaySnd(int whichIntSound, bool isLooping, float sndVol);

	bool StopSnd(int whichIntSound);
	bool StopSnd(SoundNameEnum whichSound);
	void SetSndVol(SoundNameEnum whichSound, float sndVol);
	void SetSndVol(float sndVol);

	void SetMusicVolume(float volume);

	void PlaySoundTrack(bool playIntro = false,bool forceIntro = false);
	void StopSoundTrack();

	void UpdateSoundTrack(int forceThisTrack);

protected:

	ldwSoundFx *GetSndKernal(SoundNameEnum whichSound);
	
	bool StopSndKernal(SoundNameEnum whichSound);
	void PlaySndKernal(SoundNameEnum whichSound, bool isLooping, float sndVol);

	static theSoundManager *mInstance;

private:
    
	float sfxVolume;

	ldwSoundTrack *currentSoundTrack;

	ldwSoundTrack *introSong;
	ldwSoundTrack *song1;
	ldwSoundTrack *song2;
	ldwSoundTrack *song3;
	ldwSoundTrack *song4;

	ldwSoundFx *angel;
	ldwSoundFx *pruneSnd;

	ldwSoundFx *Fountain_loop;
	ldwSoundFx *Gate_close;
	ldwSoundFx *Gate_opening;
	ldwSoundFx *Ladybug_files;
	ldwSoundFx *Moving_clay_pots;
	ldwSoundFx *Nursery_loop;
	ldwSoundFx *Rollover_1;
	ldwSoundFx *Rollover_2;
	ldwSoundFx *Rollover_3;
	ldwSoundFx *Rustle_1;
	ldwSoundFx *Rustle_2;
	ldwSoundFx *Rustle_3;
	ldwSoundFx *Seed_handling;
	ldwSoundFx *Seed_movement;
	ldwSoundFx *Seed_planting;
	ldwSoundFx *Snip_1;
	ldwSoundFx *Snip_2;
	ldwSoundFx *Snip_3;
	ldwSoundFx *Tool_swap;
	ldwSoundFx *Watering_can;
	ldwSoundFx *Windchimes_1;
	ldwSoundFx *Windchimes_2;
	ldwSoundFx *Birds;
	ldwSoundFx *Cash_register;
	ldwSoundFx *Catch_a_bug;
	ldwSoundFx *Click_1;
	ldwSoundFx *Click_2;
	ldwSoundFx *Click_3;
	ldwSoundFx *Close_trash_bin;
	ldwSoundFx *Coin;
	ldwSoundFx *Cross_breed_chime;
	ldwSoundFx *Crowd_murmer;
	ldwSoundFx *Dirt_1;
	ldwSoundFx *Dirt_2;
	ldwSoundFx *Footsteps;

	ldwSoundFx *Birds2;
	ldwSoundFx	*Buddhist_bell;
	ldwSoundFx	*Bug_move;
	ldwSoundFx	*Bug_spray;
	ldwSoundFx	*Bug_zapper;
	ldwSoundFx	*Butterfly;
	ldwSoundFx	*Cricket_loop;
	ldwSoundFx	*Cross_breed_chime_3;
	ldwSoundFx	*Gentle_fountain_loop;
	ldwSoundFx	*Insect_buzz;
	ldwSoundFx	*Medical_drip;
	ldwSoundFx	*Wind_trees;
	ldwSoundFx	*Windchimes_4;


};
#endif